unit QuicksortTestCases;

interface

uses
  TestFramework,
  Quicksort;

type
  TTestBoundaryCases = class(TTestCase)
    protected
    public
    published
      procedure TestOneItem();
      procedure TestTwoItemsInOrder();
      procedure TestTwoItemsOutOfOrder();
      procedure TestTwoItemsEqual();
  end;

  TTestCasesWith3Items = class(TTestCase)
    protected
      procedure RunTripleTest(aUnsorted : array of integer;
                              aSorted   : array of integer);
    public
    published
      procedure TestAllPermutations();
  end;

  TTestCasesWith5Items = class(TTestCase)
    protected
      procedure RunQuintupleTest(aUnsorted : array of integer;
                                 aSorted   : array of integer);
    public
    published
      procedure TestInOrder();
      procedure TestReverseOrder();
      procedure TestPivotAtFront();
      procedure TestPivotAtRear();
      procedure TestAllEqual();
    end;

implementation

procedure TTestBoundaryCases.TestOneItem();
var
  A : array [0..0] of integer;
begin
  A[0] := 42;
  QuicksortIntegers(A);
  Check(A[0] = 42, 'the only item is wrong');
end;

procedure TTestBoundaryCases.TestTwoItemsInOrder();
var
  A : array [0..1] of integer;
begin
  A[0] := 42;
  A[1] := 99;
  QuicksortIntegers(A);
  Check(A[0] = 42, 'the smaller item should be first ');
  Check(A[1] = 99, 'the larger item should be last ');
end;

procedure TTestBoundaryCases.TestTwoItemsOutOfOrder();
var
  A : array [0..1] of integer;
begin
  A[0] := 99;
  A[1] := 42;
  QuicksortIntegers(A);
  Check(A[0] = 42, 'the smaller item should be first ');
  Check(A[1] = 99, 'the larger item should be last ');
end;

procedure TTestBoundaryCases.TestTwoItemsEqual();
var
  A : array [0..1] of integer;
begin
  A[0] := 42;
  A[1] := 42;
  QuicksortIntegers(A);
  Check((A[0] = 42) and (A[1] = 42), 'both items should be equal');
end;


procedure TTestCasesWith3Items.RunTripleTest(aUnsorted : array of integer;
                                             aSorted   : array of integer);
begin
  QuicksortIntegers(aUnsorted);
  Check((aUnsorted[0] = aSorted[0]) and
        (aUnsorted[0] = aSorted[0]) and
        (aUnsorted[0] = aSorted[0]), 'the three items are out of sequence');
end;

procedure TTestCasesWith3Items.TestAllPermutations();
begin
  RunTripleTest([1, 2, 3], [1, 2, 3]);
  RunTripleTest([1, 3, 2], [1, 2, 3]);
  RunTripleTest([2, 1, 3], [1, 2, 3]);
  RunTripleTest([2, 3, 1], [1, 2, 3]);
  RunTripleTest([3, 1, 2], [1, 2, 3]);
  RunTripleTest([3, 2, 1], [1, 2, 3]);
  RunTripleTest([1, 1, 2], [1, 1, 2]);
  RunTripleTest([1, 2, 1], [1, 1, 2]);
  RunTripleTest([2, 1, 1], [1, 1, 2]);
  RunTripleTest([1, 2, 2], [1, 2, 2]);
  RunTripleTest([2, 1, 2], [1, 2, 2]);
  RunTripleTest([2, 2, 1], [1, 2, 2]);
  RunTripleTest([1, 1, 1], [1, 1, 1]);
end;


procedure TTestCasesWith5Items.RunQuintupleTest(aUnsorted : array of integer;
                                                aSorted   : array of integer);
var
  i : integer;
begin
  QuicksortIntegers(aUnsorted);
  for i := 0 to 4 do
    Check((aUnsorted[i] = aSorted[i]), 'the five items are out of sequence');
end;

procedure TTestCasesWith5Items.TestInOrder();
var
  A : array [0..4] of integer;
begin
  RunQuintupleTest([1, 2, 3, 4, 5], [1, 2, 3, 4, 5]);
end;

procedure TTestCasesWith5Items.TestReverseOrder();
var
  A : array [0..4] of integer;
begin
  RunQuintupleTest([5, 4, 3, 2, 1], [1, 2, 3, 4, 5]);
end;

procedure TTestCasesWith5Items.TestPivotAtFront();
var
  A : array [0..4] of integer;
begin
  RunQuintupleTest([5, 4, 1, 2, 3], [1, 2, 3, 4, 5]);
end;

procedure TTestCasesWith5Items.TestPivotAtRear();
var
  A : array [0..4] of integer;
begin
  RunQuintupleTest([1, 4, 5, 2, 3], [1, 2, 3, 4, 5]);
end;

procedure TTestCasesWith5Items.TestAllEqual();
var
  A : array [0..4] of integer;
begin
  RunQuintupleTest([1, 1, 1, 1, 1], [1, 1, 1, 1, 1]);
end;



initialization
  RegisterTest(TTestBoundaryCases.Suite);
  RegisterTest(TTestCasesWith3Items.Suite);
  RegisterTest(TTestCasesWith5Items.Suite);
end.
